@@ -1106,8 +1106,11 @@ def make_hashable(dataStructure, memo=None):
         reset_stubs()
         memo = {}
 
-    # If this is a re-visit, return a stub string
-    if id(dataStructure) in memo:
+    # If this is a re-visit to a container object, return a stub string
+    if (
+        not isinstance(dataStructure, (int, float, str, bool))
+    and id(dataStructure) in memo
+    ):
         return memo[id(dataStructure)]
 
     # Save a stub string in case of a future re-visit
@@ -1644,8 +1647,8 @@ def make_structure_comparator(
             # otherwise, we can check values
 
             imatch, imiss, iext = sequence_diff(
-                val.items(),
-                ref.items(),
+                list(val.items()),
+                list(ref.items()),
                 order_matters=order_matters,
                 item_matcher=item_matcher
             )
