@@ -78,6 +78,7 @@ the code is run.
 import textwrap
 import re
 import os
+import sys
 
 import pygments
 
@@ -294,6 +295,29 @@ def wrapped_with_prefix(obj, prefix, indentation_level=0):
     return textwrap.indent(rep, ' ' * indentation_level)
 
 
+def extract(context, key):
+    """
+    Gets value from a context but shows the context's error log if it
+    can't.
+    """
+    if key in context:
+        return context[key]
+    else:
+        if "ref_error_log" in context:
+            print(
+                (
+                    "Error: context is missing the '{}' key. Error log"
+                    " is:\n{}"
+                ).format(key, context["ref_error_log"]),
+                file=sys.stderr
+            )
+        else:
+            print(
+                "Error: context is missing the '{}' key. No error log"
+                " available."
+            )
+
+
 #--------------#
 # Registration #
 #--------------#
@@ -516,19 +540,18 @@ class Snippet(specifications.TestGroup):
             "<solution>"
         )
 
-        # Discard the run_for_base_and_ref_values augmentation: we don't
-        # need it since we'll be providing the solution code as the
-        # "submission".
-        if "run_for_base_and_ref_values" in self.default_augmentations:
-            del self.default_augmentations["run_for_base_and_ref_values"]
-        if "run_for_base_and_ref_values" in self.augmentations:
-            del self.augmentations["run_for_base_and_ref_values"]
+        # Tell the run_for_base_and_ref_values augmentation to run only
+        # for ref values.
+        self.augmentations.setdefault(
+            "run_for_base_and_ref_values",
+            {}
+        )["ref_only"] = True
 
         for case in self.tests:
-            if "run_for_base_and_ref_values" in case.default_augmentations:
-                del case.default_augmentations["run_for_base_and_ref_values"]
-            if "run_for_base_and_ref_values" in case.augmentations:
-                del case.augmentations["run_for_base_and_ref_values"]
+            case.augmentations.setdefault(
+                "run_for_base_and_ref_values",
+                {}
+            )["ref_only"] = True
 
         # Create our Goal object
         goal = self.provide_goal()
@@ -560,20 +583,20 @@ class Snippet(specifications.TestGroup):
                 cdict = ctx.create(base_context)
             except context_utils.ContextCreationError:
                 cdict = {
-                    # In case it's a Variables snippet
-                    "variable": "UNKNOWN",
-                    # for RunModule snippets
+                    # for RunModule snippets (note no ref_)
                     "filename": "UNKNOWN",
+                    # In case it's a Variables snippet
+                    "ref_variable": "UNKNOWN",
                     # for FunctionCalls snippets
-                    "function": "UNKNOWN",
-                    "args": (),
-                    "kwargs": {},
+                    "ref_function": "UNKNOWN",
+                    "ref_args": (),
+                    "ref_kwargs": {},
                     # for Blocks snippets
-                    "block": "UNKNOWN",
+                    "ref_block": "UNKNOWN",
                     # For all kinds of snippets
-                    "value": "There was an error compiling this snippet.",
-                    "output": "There was an error compiling this snippet.",
-                    "error_log": html_tools.string_traceback()
+                    "ref_value": "There was an error compiling this snippet.",
+                    "ref_output": "There was an error compiling this snippet.",
+                    "ref_error_log": html_tools.string_traceback()
                 }
             context_dicts.append(cdict)
 
@@ -583,14 +606,20 @@ class Snippet(specifications.TestGroup):
             base_context["task_info"]["specification"].soln_path
         )
         for context in context_dicts:
-            if "output" in context and soln_path in context["output"]:
-                context["output"] = context["output"].replace(
+            if (
+                "ref_output" in context
+            and soln_path in context["ref_output"]
+            ):
+                context["ref_output"] = context["ref_output"].replace(
                     soln_path,
                     '&lt;soln&gt;'
                 )
 
-            if "error_log" in context and soln_path in context["error_log"]:
-                context["error_log"] = context["error_log"].replace(
+            if (
+                "ref_error_log" in context
+            and soln_path in context["ref_error_log"]
+            ):
+                context["ref_error_log"] = context["ref_error_log"].replace(
                     soln_path,
                     '&lt;soln&gt;'
                 )
@@ -628,12 +657,12 @@ class Snippet(specifications.TestGroup):
         `render_context` but also potentially adding additional HTML for
         extra context slots (see e.g., `show_image` and `play_audio`).
         """
-        if not self.ignore_errors and "error" in context:
+        if not self.ignore_errors and "ref_error" in context:
             logging.log(
                 "Error captured by context creation:\n"
-              + context["error"]
+              + context["ref_error"]
               + "\nFull log is:\n"
-              + context.get("error_log", '<missing>')
+              + context.get("ref_error_log", '<missing>')
             )
         result = self.render_context(context)
         for glamer in self.glamers:
@@ -667,6 +696,9 @@ class Snippet(specifications.TestGroup):
         that context gets rendered. The editor may mutate the context it
         is given in order to change what gets rendered. Multiple calls to
         this function each simply replace the previous editor function.
+
+        Note that the ref_* values from the context are used to create
+        the snippet.
         """
         self.editor = editor
 
@@ -674,8 +706,8 @@ class Snippet(specifications.TestGroup):
         """
         Adds a glamer that augments output by appending a "Image "
         prompt followed by an image element which shows the contents of
-        the "image" context slot. To establish an "image" slot, call a
-        method like
+        the "ref_image" context slot. To establish a "ref_image" slot,
+        call a method like
         `potluck.specifications.HasPayload.capture_turtle_image`.
 
         Returns self for chaining.
@@ -687,8 +719,9 @@ class Snippet(specifications.TestGroup):
         """
         Adds a glamer that augments output by appending an "Audio "
         prompt followed by an audio element which plays the contents of
-        the "audio" context slot. To make an "audio" slot, call a method
-        like `potluck.specifications.HasPayload.capture_wavesynth`.
+        the "ref_audio" context slot. To make an "ref_audio" slot, call a
+        method like
+        `potluck.specifications.HasPayload.capture_wavesynth`.
 
         Returns self for chaining.
         """
@@ -699,9 +732,9 @@ class Snippet(specifications.TestGroup):
         """
         Adds a glamer that augments output by appending a "File"
         prompt followed by a text box showing the filename and the
-        contents of that file. Requires 'output_filename' and
-        'output_file_contents' slots, which could be established using a
-        method like
+        contents of that file. Requires 'ref_output_filename' and
+        'ref_output_file_contents' slots, which could be established
+        using a method like
         `potluck.specifications.HasPayload.capture_file_contents`
 
         Returns self for chaining.
@@ -764,8 +797,8 @@ class Variables(Snippet):
         the definition of a variable, including Jupyter-notebook-style
         prompts.
         """
-        varname = context["variable"]
-        value = context["value"]
+        varname = extract(context, "ref_variable")
+        value = extract(context, "ref_value")
 
         # format value's repr using wrapped_repr, but leaving room for
         # varname = at beginning of first line
@@ -803,9 +836,9 @@ class RunModule(Snippet):
         running a module. Includes a Jupyter-style prompt with %run magic
         syntax to show which file was run.
         """
-        filename = context["filename"]
-        captured = context.get("output", '')
-        captured_errs = context.get("error_log", '')
+        filename = extract(context, "filename") # Note: no ref_ here
+        captured = context.get("ref_output", '')
+        captured_errs = context.get("ref_error_log", '')
 
         # Wrap faked inputs with spans so we can color them blue
         captured = re.sub(
@@ -869,15 +902,19 @@ class FunctionCalls(Snippet):
         calling a function. Includes a Jupyter-style prompt for input as
         well as the return value.
         """
-        if "error" in context:
-            print("Error during context creation:")
-            print(context["error"])
-        fname = context["function"]
-        value = context["value"]
-        args = context["args"]
-        kwargs = context["kwargs"]
-        captured = context.get("output", '')
-        captured_errs = context.get("error_log", '')
+        if "ref_error" in context:
+            print(
+                "Error during context creation:"
+              + context["ref_error"]
+              + "\nFull log is:\n"
+              + context.get("ref_error_log", '<missing>')
+            )
+        fname = context["ref_function"]
+        value = context["ref_value"]
+        args = context["ref_args"]
+        kwargs = context["ref_kwargs"]
+        captured = context.get("ref_output", '')
+        captured_errs = context.get("ref_error_log", '')
 
         # Figure out representations of each argument
         argreps = []
@@ -997,10 +1034,10 @@ class Blocks(Snippet):
         input as well as the result value of the last line of the block
         as long as that's not None.
         """
-        src = context["block"]
-        value = context["value"]
-        captured = context.get("output", '')
-        captured_errs = context.get("error_log", '')
+        src = extract(context, "ref_block")
+        value = extract(context, "ref_value")
+        captured = context.get("ref_output", '')
+        captured_errs = context.get("ref_error_log", '')
 
         # Wrap faked inputs with spans so we can color them blue
         captured = re.sub(
@@ -1057,12 +1094,13 @@ class Fakes(Snippet):
         A snippet ID, title, and caption are required, along with a list
         of fake context dictionary objects to be rendered as if they came
         from a real test. Each dictionary must contain a "code" slot, and
-        may contain "value", "output", and/or "error_log" keys. If using
-        glamers, relevant keys should be added directly to these fake
-        contexts.
+        may contain "ref_value", "ref_output", and/or "ref_error_log"
+        keys. If using glamers, relevant keys should be added directly to
+        these fake contexts.
 
-        If "value" or "output" slots are missing, defaults of None and ''
-        will be added, since these slots are ultimately required.
+        If "ref_value" or "ref_output" slots are missing, defaults of
+        None and '' will be added, since these slots are ultimately
+        required.
 
         A postscript is optional (see `Snippet`).
         """
@@ -1073,10 +1111,10 @@ class Fakes(Snippet):
                     "Fake context dictionaries must contain a 'code'"
                     " slot."
                 )
-            if 'value' not in ctx:
-                ctx['value'] = None
-            if 'output' not in ctx:
-                ctx['output'] = ''
+            if 'ref_value' not in ctx:
+                ctx['ref_value'] = None
+            if 'ref_output' not in ctx:
+                ctx['ref_output'] = ''
         super().__init__(sid, title, caption, [], postscript)
 
     def create_goal(self):
@@ -1112,10 +1150,10 @@ class Fakes(Snippet):
         input as well as the result value of the last line of the block
         as long as that's not None.
         """
-        src = context["code"]
-        value = context["value"]
-        captured = context.get("output", '')
-        captured_errs = context.get("error_log", '')
+        src = extract(context, "code")
+        value = extract(context, "ref_value")
+        captured = context.get("ref_output", '')
+        captured_errs = context.get("ref_error_log", '')
 
         # Wrap faked inputs with spans so we can color them blue
         captured = re.sub(
@@ -1252,8 +1290,8 @@ class Files(Snippet):
         displaying a file with %more magic. Includes a Jupyter-style
         prompt for input as well as the contents of the file.
         """
-        path = context["path"]
-        contents = context["contents"]
+        path = extract(context, "path")
+        contents = extract(context, "contents")
 
         result = (
             '<span class="prompt input">In []:</span>'
@@ -1274,25 +1312,26 @@ class Files(Snippet):
 
 def append_image_result(markup, context):
     """
-    Given some HTML markup and a context dictionary, turns the "image"
-    context slot into an HTML img tag and returns the given markup with
-    that appended, prefixed by an "Image " prompt. If the context has
-    no "image" value, the original markup is returned unmodified.
+    Given some HTML markup and a context dictionary, turns the
+    "ref_image" context slot into an HTML img tag and returns the given
+    markup with that appended, prefixed by an "Image " prompt. If the
+    context has no "image" value, the original markup is returned
+    unmodified.
 
-    The "image_alt" slot of the context is used as alt text for the
-    image, with the "output" slot being used as backup (under a hopeful
-    assumption about turtleBeads being imported for printed
-    descriptions). If neither is present, "no alt text available" will
-    be used.
+    The "ref_image_alt" slot of the context is used as alt text for the
+    image, with the "ref_output" slot being used as backup (under a
+    hopeful assumption about turtleBeads being imported for printed
+    descriptions). If neither is present, "no alt text available" will be
+    used.
     """
     # Short-circuit unless we've got an image
-    if "image" not in context:
+    if "ref_image" not in context:
         return markup
 
-    image = context["image"]
+    image = extract(context, "ref_image")
     alt = context.get(
-        "image_alt",
-        context.get("output", "no alt text available")
+        "ref_image_alt",
+        context.get("ref_output", "no alt text available")
     )
 
     img_tag = html_tools.html_image(image, alt, ["example"])
@@ -1305,9 +1344,9 @@ def append_image_result(markup, context):
 
 def append_audio_result(markup, context):
     """
-    Given some HTML markup and a context dictionary, turns the "audio"
-    context slot into an HTML audio tag and returns the given markup with
-    that appended, prefixed by an "Audio " prompt.
+    Given some HTML markup and a context dictionary, turns the
+    "ref_audio" context slot into an HTML audio tag and returns the given
+    markup with that appended, prefixed by an "Audio " prompt.
 
     Note that this can result in a pretty large string if the WAV format
     is used, since the string needs to be base64-encoded and WAV is
@@ -1316,20 +1355,20 @@ def append_audio_result(markup, context):
 
     TODO: Maybe use Ogg/Vorbis?
 
-    If there is no "audio" context slot, the given markup is returned
+    If there is no "ref_audio" context slot, the given markup is returned
     unmodified.
 
-    The "audio" context value must be a dictionary with at least
-    "mimetype" and "data" slots containing the MIME type for the data
-    and the data itself (as a bytes object). It may also include a
-    "label" slot which will be used invisibly as an aria-label property
-    of the audio element; if absent no aria-label will be attached.
+    The "ref_audio" context value must be a dictionary with at least
+    "mimetype" and "data" slots containing the MIME type for the data and
+    the data itself (as a bytes object). It may also include a "label"
+    slot which will be used invisibly as an aria-label property of the
+    audio element; if absent no aria-label will be attached.
     """
     # Short-circuit unless we've got audio
-    if "audio" not in context:
+    if "ref_audio" not in context:
         return markup
 
-    audio = context["audio"]
+    audio = extract(context, "ref_audio")
     mime = audio["mimetype"]
     data = audio["data"]
     label = audio.get("label")
@@ -1345,24 +1384,24 @@ def append_audio_result(markup, context):
 def append_file_contents(markup, context):
     """
     Given some HTML markup and a context dictionary, turns the
-    "output_filename" and "output_file_contents" context slots into HTML
-    markup displaying the contents of that file, which gets appended to
-    the given markup and returned.
+    "ref_output_filename" and "ref_output_file_contents" context slots
+    into HTML markup displaying the contents of that file, which gets
+    appended to the given markup and returned.
 
     Note that this is intended to display the contents of text files.
 
-    If either of the "output_filename" or "output_file_contents" context
-    slots are missing, the given markup is returned unmodified.
+    If either of the "ref_output_filename" or "ref_output_file_contents"
+    context slots are missing, the given markup is returned unmodified.
     """
     # Short-circuit unless we've got output file contents
     if (
-        "output_filename" not in context
-     or "output_file_contents" not in context
+        "ref_output_filename" not in context
+     or "ref_output_file_contents" not in context
     ):
         return markup
 
-    filename = context["output_filename"]
-    contents = context["output_file_contents"]
+    filename = extract(context, "ref_output_filename")
+    contents = extract(context, "ref_output_file_contents")
 
     return (
         markup
