@@ -25,9 +25,6 @@ def whatever(a, b):
 
 import optimism as opt
 
-# Not enough expectations (3 required)
-opt.testCase(whatever(True, True))
-opt.expectResult("Both")
+opt.expect(whatever(True, True), "Both")
 # Expectation crashes:
-opt.testCase(whatever(False, False))
-opt.expectResult("C")
+opt.expect(whatever(False, False), "C")
