@@ -13,6 +13,7 @@ __version__ = "0.2"
 import sys, os, shutil, subprocess, threading, copy
 import time, datetime
 import base64, csv
+import shlex
 
 from flask import json
 
@@ -100,8 +101,7 @@ def evaluation_directory(course, semester):
     The evaluation directory for a particular class/semester.
     """
     return os.path.join(
-        os.getcwd(),
-        _CONFIG["EVALUATION_BASE"],
+        _EVAL_BASE,
         course,
         semester
     )
@@ -1906,11 +1906,14 @@ def init(config, key=None):
     should exist unless a key is given (should be a byte-string). If
     'redis-pw.conf' doesn't exist, it will be created.
     """
-    global _REDIS, _CONFIG
+    global _REDIS, _CONFIG, _EVAL_BASE
 
     # Store config object
     _CONFIG = config
 
+    # Compute evaluation base directory based on init-time CWD
+    _EVAL_BASE = os.path.join(os.getcwd(), _CONFIG["EVALUATION_BASE"])
+
     # Grab port from config
     port = config.get("STORAGE_PORT", 51723)
 
@@ -2053,7 +2056,23 @@ def init(config, key=None):
 
         # Try to start a new redis server...
         print("...starting Redis...")
-        subprocess.Popen(["redis-server", rconf_file])
+        try:
+            subprocess.Popen(["redis-server", rconf_file])
+        except OSError:
+            # If running through e.g. Apache and this command fails, you
+            # can try to start it manually, so we point that out
+            if len(shlex.split(rconf_file)) > 1:
+                rconf_arg = "'" + rconf_file.replace("'", r"\'") + "'"
+            else:
+                rconf_arg = rconf_file
+            sys.stdout.write(
+                (
+                    "Note: Failed to start redis-server with an"
+                    " OSError.\nYou could try to manually launch the"
+                    " server by running:\nredis-server {}"
+                ).format(rconf_arg)
+            )
+            raise
         time.sleep(0.2) # try to connect pretty quickly
         print("...we hope Redis is up now...")
 
